/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it
 * is applied to this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * AgroSense. If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.task.shape;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import nl.cloudfarming.client.field.shape.ShapeDataObject;
import nl.cloudfarming.client.field.shape.TreatmentZoneImportCookie;
import org.openide.awt.ActionID;
import org.openide.awt.ActionReference;
import org.openide.awt.ActionReferences;
import org.openide.awt.ActionRegistration;
import org.openide.nodes.Node;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle.Messages;
import org.openide.util.actions.NodeAction;

@ActionID(category = "Task", id = "nl.cloudfarming.client.task.shape.CreateTaskFromShapeAction")
@ActionRegistration(displayName = "#CTL_CreateTaskFromShape", lazy = false)
@ActionReferences(value = {
    @ActionReference(path = "Loaders/application/x-esri-shape/Actions", position = 3000)})
@Messages({"CTL_CreateTaskFromShape=Convert to task"})
public final class CreateTaskFromShapeAction extends NodeAction {

    public CreateTaskFromShapeAction() {
    }

    @Override
    protected void performAction(Node[] activatedNodes) {
        for (Node node : activatedNodes) {
            TreatmentZoneImportCookie cookie = node.getLookup().lookup(TreatmentZoneImportCookie.class);
            List<TreatmentZoneImportCookie> cookies = new ArrayList<>();
            cookies.add(cookie);
            ExecutorService executor = Executors.newFixedThreadPool(1);
            TaskCreator creator = new TaskCreator(cookies);
            executor.execute(creator);
            executor.shutdown();
        }
    }

    @Override
    protected boolean enable(Node[] activatedNodes) {
        for (Node node : activatedNodes) {
            TreatmentZoneImportCookie cookie = node.getLookup().lookup(TreatmentZoneImportCookie.class);
            if (cookie == null) {
                return false;
            }
            // TODO: task creator now filters on valid shapes; invert predicate? 
            // (i.e. only refuse a shapefile if the number of valid shapes is 0)
            // check could be moved into SDO itself while setting the import cookies.
            ShapeDataObject dataObject = cookie.getShapeDataObject();
            if (dataObject.getNumberOfInvalidShapes() > 0) {
                return false;
            }

        }
        return true;
    }

    @Override
    public String getName() {
        return Bundle.CTL_CreateTaskFromShape();
    }

    @Override
    public HelpCtx getHelpCtx() {
        return HelpCtx.DEFAULT_HELP;
    }
}
